#include "courselib.h"
#include <cmath>
#include "TH1.h"
#include "TFile.h" 
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <iostream>
using namespace std;

/*
	Testing Monte Carlo integration.
	We have a test function:  g0=(1-x)**pow1/x
      We weight the histo with the function value.
	It is integrated over the region from xmin to 1.

	We calculate the result and its error.
      
	The random number generator is RANLUX by M. Luescher.
      M. Lscher, A portable high-quality random number generator 
      for lattice field theory simulations, 
      Computer Physics Communications 79 (1994) 100  
*/


double g0(double z)
{	
	double d1,pow1=4,result;
	d1 = 1-z;
      result=pow(d1,pow1)/z;
	return(result); 
}

int main(int argc,char **argv)
{
      TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);
	int n1, npoints = 10000000;
 	double x0, xmin=1e-4, weight;
      double xg0, xg00, f, ff, sigma2, error;
#define LVEC 1
	double rvec[LVEC];
// initialise random number generator
//    rlxd_init( luxory level, seed )
      rlxd_init(2,32767);
//               TH1F("label","title",nr of bins, xlow,xhigh )
      TH1F *histo1 = new TH1F("x","g(x,mu0)",100, -5, 1.);
      
      xg0 = xg00 = 0;
      
	for ( n1 = 0; n1 < npoints; n1++ ) {
            ranlxd(rvec,LVEC);
//   here do the calcualtion with importance sampling
		x0 = pow(xmin,rvec[0]);
            weight = x0*log(1/xmin) ;
// here do the calcualtion using linear sampling
//		x0 = xmin+(1-xmin)*rvec[0];
//            weight = 1-xmin ;
            f=g0(x0); 
            ff = weight*f;
            xg0 = xg0 + ff;
            xg00 = xg00 + ff*ff; 
//                                  
	}

      xg0 = xg0/npoints;
      xg00 = xg00/npoints;
      sigma2 = xg00 - xg0*xg0 ;
      error = sqrt(sigma2/npoints) ;
      cout<<" integral for g(x) = (1-x)**4/x is: "<<xg0<<"+/-"<< error<<endl;
	return(0);
}

